#include "sixsitejump.h"

#include <iomanip>
#include <iostream>

#include "coordinates.h"

namespace motions {
    SixSiteOctahedronC3::SixSiteOctahedronC3(const double delta, const double eta, const double chi, std::mt19937_64& rng) :
        m_chi{chi*M_PI/180.},
        BaseMotion(std::string{"SixSiteOctahedral"}, delta, eta, rng) {}

    SixSiteOctahedronC3::SixSiteOctahedronC3(std::mt19937_64& rng) : BaseMotion(std::string{"SixSiteOctahedralC3"}, rng) {}

    void SixSiteOctahedronC3::initialize() {
        const coordinates::SphericalPos c3_axis = draw_position();
        const auto [x, y, z] = spherical_to_xyz(c3_axis);
        const double alpha = 2. * M_PI * m_uni_dist(m_rng);

        const double m_chi_opposite = M_PI - m_chi;

        for (int i = 0; i<3; i++) {
            m_corners[2*i] = omega_q(rotate(c3_axis, m_chi, alpha + i * 2./3.*M_PI));
            m_corners[2*i+1] = omega_q(rotate(c3_axis, m_chi_opposite, alpha + i * 2./3.*M_PI + M_PI/3.));
        }

        m_initial_omega = SixSiteOctahedronC3::jump();
    }


    double SixSiteOctahedronC3::jump() {
        m_corner_idx += m_chooser(m_rng);
        m_corner_idx %= 6;

        return m_corners[m_corner_idx];
    }

    std::string SixSiteOctahedronC3::toString() const {
        return {"SixSiteOctahedral/angle=" + std::to_string(m_chi / M_PI * 180.)};
    }
}
